function [p,iter] = CuthillMcKee(A)
%
% function [p,iter] = CuthillMcKee(A)
% ===================================
%
%  Implements Cuthill-McKee to find permuation sequence p 
%  that will minimize bandwidth of A.  
%  ( iter is the different starting indices used )

%  (c) 2010 by Tomas Co
%  @ Michigan Technological University

    B           = logical(sparse(A))                    ;
    B           = B|B'                                  ;
    N           = size(B,1)                             ;
    [z,s]       = min(sum(B))                           ;
    p           = cuthill(B,s)                          ;
    wd          = bandwidth(A(p,p))                     ;
    iter        = 1                                     ;
    tried       = [s]                                   ;
    lvs         = LevStruct(A,p)                        ;
    qs          = p( lvs(end):N )                       ;
    while iter <= N                                     ;
      brk = true                                        ;
      for s=qs                                          ;
        if ~any(tried==s)                               ;
            tried       = [tried,s]                     ;                
            iter        = iter + 1                      ;
            pnew        = cuthill(A,s)                  ;
            wdnew       = bandwidth(A(pnew,pnew))       ;
            lvs         = LevStruct(A,pnew)             ;
            if wdnew<wd                                 ;
                p       = pnew                          ;
                wd      = wdnew                         ;
                qs      = p( lvs(end):N )               ;
                brk     = false                         ;
            end                                         ;
        end                                             ;
      end                                               ;
      if brk                                            ;
            break                                       ;
      end                                               ;
    end                                                 ;
    p           = fliplr(p)                             ;
    
    function V = cuthill(A,s)
        % (c) 2010 by Tomas Co
        N           = size(A,1)                         ;
        Ent         = s                                 ;
        Rorig       = 1:N                               ;
        V           = []                                ;
        cnt         = 0                                 ;
        visited     = sparse(false(N,1))                ;
        deg         = sum(A)                            ;
        while ( ~all(visited) )                         ;
            visited(Ent)= true                          ;
            cnt         = cnt + 1                       ;   
            V           = [V,Ent]                       ;
            Eind        = A(:,V(cnt))&(~visited)        ;
            Nesub       = Rorig( Eind )                 ;
            nxtdeg      = deg( Eind )                   ;
            [z,nn]      = sort(nxtdeg)                  ;
            Ent         = Nesub(nn)                     ;
        end                                             ;
        V       = [V,Ent]                               ;
        
     function levs = LevStruct(A,P)
        % (c) 2010 by Tomas Co
        B           = tril( A(P,P) )                    ;
        N           = size(A,1)                         ;
        lw          = 1                                 ;
        ls          = 1                                 ;
        ind         = [1]                               ;
        levs        = [1]                               ;
        while ls+lw <= N                                ;
            ls          = ls+lw                         ;
            levs        = [levs,ls]                     ;
            m           = size(B,1)                     ;
            q           = B(lw+1:m,1:lw)                ;
            if lw>1
                q       = any(q')'                      ;
            end
            lw          = full(sum(q))                  ;
            B = B(lw+1:m,lw+1:m)                        ;
        end                                             ;

    function bw = bandwidth(A)
        %  (c) 2010 by Tomas Co
        [i,j]       = find(tril(A))                     ;
        bw          = max(i-j)                          ;