function    [x,f] = NewtonMin( Fx, x0, type, par )
%                                                                        ;
%   function [x,f] = NewtonMin( Fx, x0, type, par )                        ;
%   ================================================                     ;
%   Enhanced-Newton Minimization:                                           ;
%       Fx  : handle for function                                        ;
%       x0  : initial guess                                              ;
%       type: 1=line search (default) , 2=dogleg                         ;
%       par : parameters (optional)                                      ;
%  (adapted from: Dennis & Schnabel,"Numerical Methods for Uncon-        ;
%   strained Optimization and Nonlinear Equations",Prentice Hall,1983)   ;

%   (c) 2007 by Tomas Co                                                 ;
%   @ Michigan Technological Universtity                                 ;

Gtol = (eps)^(0.33) ;   StpTol = Gtol^2   ;   MXstep = 1000              ;
MxIter = 1e6        ;   nmax = 0          ;   xc = x0                    ;
if  nargin<3 ;      type = 1              ;   end                        ;
if  nargin>3 ;      parY = true ; else    ;   parY = false      ;  end   ;
if  parY ; fc=feval(Fx,xc,par)  ; gc=Grad(Fx,xc,par); hc=Hess(Fx,xc,par) ;
else     ; fc=feval(Fx,xc)      ; gc=Grad(Fx,xc)    ; hc=Hess(Fx,xc)     ;
end                                                                      ;
[chk,nmax] = CKend( xc,fc,gc,Gtol,nmax )                                 ;
if chk == 1         ;   x = xc    ;     f = fc  ;   return      ;  end   ;
not_done = true     ;   iter = 0  ;     del = -1                         ;
while not_done                                                           ;
  [p,Lc] = pchol(hc,-gc)          ;     iter = iter + 1                  ;
  if  type==1                                                            ;
    if parY; [x,f,NoCh,MXed]=lines(Fx,xc,fc,gc,p,MXstep,StpTol,false,par);
    else   ; [x,f,NoCh,MXed]=lines(Fx,xc,fc,gc,p,MXstep,StpTol,false)    ;
    end                                                                  ;
  else                                                                   ;
    if parY; [x,f,del,NoCh,MXed]=dogleg(Fx,xc,fc,gc,Lc,p,del,MXstep,...
                                                       StpTol,false,par) ;
    else   ; [x,f,del,NoCh,MXed]=dogleg(Fx,xc,fc,gc,Lc,p,del,MXstep,...
                                                       StpTol,false)     ;
    end                                                                  ;
  end                                                                    ;
  if parY  ;     g = Grad(Fx,x,par)  ;  else   ; g = Grad(Fx,x)   ; end  ;                                                        ;
  [chk,nmax] = CKend(x,f,g,Gtol,nmax,xc,iter,NoCh,MXed,StpTol,MxIter)    ;
  xc = x   ;     fc = f              ;  gc = g                           ;
  if parY  ;     hc = Hess(Fx,xc,par);  else   ; hc = Hess(Fx,xc) ; end  ;
  if chk>0 ;     not_done = false    ;  end                              ;
end                                                                      ;
%                                                                        ;
  function    [x,L] = pchol(H, b)                                        ;
% ===============================                                        ;
%       solution of Hx=b via perturbed Cholesky factorization            ;
%                                                                        ;
    n = size(H,2)   ;   L = zeros(n,n)      ;   v = zeros(n,1)           ;
    MXoff = max(1, max(abs(diag(H))) )      ;   MNv = eps^(1/2)*MXoff    ;
    for j=1:n                                                            ;
        jn = j:n        ;   j1 = 1:j-1                                   ;
        if  j>1         ;   v =L(jn,j1)*L(j,j1)'    ;   end              ;
        v = H(jn,j)-v   ;   a = sqrt( max([(v.*v)/MXoff;MNv;v(1)]) )     ;
        v = v/a         ;   v(1)=a                  ;   L(jn,j)=v        ;
    end                                                                  ;
    y = b(1)/L(1,1)                                                      ;
    for i = 2:n     ;  y = [y;(b(i)-L(i,1:i-1)*y)/L(i,i)]   ; end        ;
    R = L'          ;  x = y(n)/R(n,n)                      ; n1 = n-1   ;
    for i=n1:-1:1   ;  x = [(y(i)-R(i,i+1:n)*x)/R(i,i);x]   ; end        ;
    
function [chk,nmax]=CKend(x,f,g,Gtol,nmax,xc,iter,NoCh,MXed,StpTol,MxIter)
% ========================================================================
%       check termination                                                ;
%                                                                        ;
v1 = max(abs(x),1); v3 = max(abs(g').*v1/max(abs(f),1))                  ;
if  v3<=Gtol*1e-3 ; chk = 1;disp(' Tolerance met for gradient'); return  ;
else ;              chk = 0                                              ;
end                                                                      ;
if nargin > 5                                                            ;
    v4 = max([abs(x-xc)./v1])                                            ;
    if NoCh;               chk = 3 ; disp('No change in global steps ')  ;
    elseif v4<=StpTol  ;   chk = 2 ; disp('Tolerance met for step size') ;
    elseif iter>MxIter ;   chk = 4 ; disp('Number iterations exceeded')  ;
    elseif  MXed       ;   nmax = nmax + 1                               ;
        if nmax>4 ; chk = 5 ; disp('Maxed 5 consecutive times') ;  end   ;
    else               ;   nmax =0                                       ;
    end                                                                  ;
end                                                                      ;
    
function   g = Grad(Fx,x,par)                                            ;
% ===========================                                            ;
d=eps^(0.5)  ;  h=max(abs(x'),1)*d  ;  g=[]  ; n=length(x)               ;
if nargin>2  ;  gr=feval(Fx,x,par)  ;  else  ; gr=feval(Fx,x) ;  end     ;
for i=1:n                                                                ;
  xx=x       ;  xx(i)=x(i)+h(i)                                          ;
  if nargin>2;  gg=feval(Fx,xx,par) ;  else  ; gg=feval(Fx,xx);  end     ;
  g=[g,(gg-gr)/h(i)]                                                     ;
end                                                                      ;

function    H   = Hess(Fx,x,par)                                         ;
% ==============================                                         ;
d=eps^(0.3)     ;      n=length(x)          ;  H=zeros(n,n)              ;
h  = max(abs(x),1)*d                        ;  h2 = h/2                  ;
if nargin>2                                                              ;
    f = feval(Fx,x,par)                                                  ;
    for i=1:n                                                            ;
        hi = h(i) ; f_p=rsub(Fx,x,h,i,par)  ; f_m=rsub(Fx,x,h,-i,par)    ;
        H(i,i) = (f_p - 2*f + f_m)/(hi^2)                                ;
        for j=i+1:n                                                      ;
          hj   = h(j)                                                    ;
          f_pp =rsub(Fx,x,h2,[ i, j],par);f_mp =rsub(Fx,x,h2,[-i, j],par);
          f_pm =rsub(Fx,x,h2,[ i,-j],par);f_mm =rsub(Fx,x,h2,[-i,-j],par);
          H(i,j)=(f_pp-f_mp-f_pm+f_mm)/(hi*hj)  ;  H(j,i)=H(i,j)         ;
        end                                                              ;
    end                                                                  ;
else                                                                     ;
    f = feval(Fx,x)                                                      ;
    for i=1:n                                                            ;
        hi = h(i) ; f_p=rsub(Fx,x,h,i)      ; f_m=rsub(Fx,x,h,-i)        ;
        H(i,i) = (f_p - 2*f + f_m)/(hi^2)                                ;
        for j=i+1:n                                                      ;
          hj   = h(j)                                                    ;
          f_pp =rsub(Fx,x,h2,[ i, j])    ; f_mp =rsub(Fx,x,h2,[-i, j])   ;
          f_pm =rsub(Fx,x,h2,[ i,-j])    ; f_mm =rsub(Fx,x,h2,[-i,-j])   ;
          H(i,j)=(f_pp-f_mp-f_pm+f_mm)/(hi*hj)  ;  H(j,i)=H(i,j)         ;
        end                                                              ;
    end                                                                  ;
end                                                                      ;

function    f = rsub(Fx,x,h,ind,par)                                     ;
% ==================================                                     ;
for p=ind   ;   j = abs(p)          ;   x(j)=x(j)+h(j)*sign(p)  ;  end   ;
if nargin>4 ;   f = feval(Fx,x,par) ;   else  ; f = feval(Fx,x) ;  end   ;