function    u = dynamic_2d_fem(P,Ind,...
                  Mhandle,bhandle,ghandle,hhandle,chandle,...
                  DBChandle,NBChandle,uinit,dt,tinitial,tfinal)
%
%function    u = dynamic_2d_fem(P,Ind,...
%                  Mhandle,bhandle,ghandle,hhandle,chandle...
%                  DBChandle,NBChandle,uinit,dt,tinitial,tfinal)
%  =============================================================
%
%    PDE:
%       div( M dot grad(u)) + b dot grad(u) + gu + h = 0
%
%    P       : node positions        [=] 2 x Nnodes
%    Ind     : indices of elements   [=] Nelements x 3
%    @Mhandle,@bhandle,
%      @ghandle,@hhandle,
%      @chandle :                   function handles
%    @DBChandle, @NBChandle:     boundary conditions
%          should yield: 
%                       DBC (Dirichlet nodes)
%                       DBCval (Dirichlet values)
%                       NBC (Neumann nodes)
%                       NBCval (Neumann values)
%    uinit   : initial values
%    dt      : time increment
%    tfinal  : final time
%
%   (assumes that all the triangles have been indexed
%         with a counter-clockwise sequence)

%       (c) 2011 by Tomas Co
%       Michigan Technological University
%
    Nnodes          = size(P,2)                     ;
    Nelements       = size(Ind,1)                   ;
    K               = sparse(Nnodes,Nnodes)         ;
    GM              = sparse(Nnodes,1)              ;
    S1              = sparse(Nnodes,1);
    S2              = sparse(Nnodes,1);
    C               = sparse(Nnodes,Nnodes)         ;    
    u               = uinit                         ;
    for t = tinitial+dt:dt:tfinal
        tau = t-dt/2                                ;
        [DBC,DBCval]    = DBChandle(P,tau)          ;
        [NBC,NBCval]    = NBChandle(P,tau)          ;
        for i=1:Nelements
            I           = Ind(i,:)                  ;
            P_loc       = P(:,I)                    ;
            NBC_loc     = NBC(I)                    ;
            NBCval_loc  = NBCval(I)                 ;
            [K_loc,Gamma_loc,C_loc] = ...
                local_fem(P_loc, Mhandle, bhandle, ... 
                ghandle, hhandle, chandle, NBC_loc, ...
                NBCval_loc, tau);
            K(I,I)      = K(I,I) + K_loc            ;
            C(I,I)      = C(I,I) + C_loc            ;
            GM(I)       = GM(I) + Gamma_loc         ;
        end
        CpK     = C + dt*K/2                        ;
        CmK     = C - dt*K/2                        ;
        for i=1:Nnodes
            if DBC(i)
                u(i) = DBCval(i)                    ;
            end
        end
        du      = CpK\(CmK*u)                       ;
        B       = CpK\(dt*GM)                       ;
        u       = du + B                            ;
    end
    [DBC,DBCval]    = DBChandle(P,t)                ;
    for i=1:Nnodes
        if DBC(i)
            u(i) = DBCval(i)                        ;
        end
    end
    %
% ===================================================
%
    function [K_n,Gamma_n,C_n]=local_fem(p,Mhandle,...
                bhandle,ghandle,hhandle,...
                chandle,NBC,NBCval,t)
        D       = det([1 1 1;p])                    ;
        T       = [0,-1;1,0]*p*...
                    [0,1,-1;-1,0,1;1,-1,0]/D        ;
        zeta    = [1;1;1]/3                         ;
        pstar   = p*zeta                            ;
        pxstar  = pstar(1)                          ;
        pystar  = pstar(2)                          ;
        M       = feval(Mhandle,pxstar,pystar,t)    ;
        b       = feval(bhandle,pxstar,pystar,t)    ;
        g       = feval(ghandle,pxstar,pystar,t)    ;
        h       = feval(hhandle,pxstar,pystar,t)    ;
        c       = feval(chandle,pxstar,pystar,t)    ;
        Q       = [0;0;0]                           ;
        if sum(NBC)>1
            if sum(NBC)>2
                disp(['All 3 nodes of element ',...
                  num2str(i),...
                  ' are attached to the boundary']) ;
                return
            end
            nn      = [1;2;3]                       ;
            switch (nn(NBC==0))
                case 1          % [0 1 1]
                    eb = [0;0;1] ; ea = [0;1;0]     ;
                case 2          % [1 0 1]
                    eb = [1;0;0] ; ea = [0;0;1]     ;
                case 3          % [1 1 0]
                    eb = [0;1;0] ; ea = [1;0;0]     ;
            end
            L   = norm(p*(eb-ea))                   ;
            Q   = [ea eb]*[ea eb]'*NBCval*L/D       ;
        end
        K_n     = (D/2)*...
                (T'*M*T - zeta*b'*T - g*zeta*zeta')  ;
        Gamma_n = (D/2)*(h*zeta+Q)                   ;
        C_n     = (D/2)*c*zeta*zeta'                 ;