function [x,y,u] = dynamic_fem_test1(dt,tsnaps)
%   function [x,y,u] = dynamic_fem_test1(dt,tsnaps)
%   =================================================
%   Dynamic FEM test for square domain
%   dt = time increment used in time marching
%   tsnaps = 9 time instants when solutions are plotted
%   

%   (c) 2011 Tomas Co
%   Michigan Technological University
%
%    
%   Obtain Mesh Data and Initial Data
%   ---------------------------------
    [x,y,Ind]   = squared_mesh_example(21)      ;   
    x           = (x+1)/2                       ;   
    y           = (y+1)/2                       ;   % triangulation
    P           = [x;y]                         ;   % node positions
    u           = get_initial_condition(x,y)    ;   % initial condition
    %
    %   Implement FEM at 9 time points
    %   ------------------------------
    %
    k = 0;
    oldt = 0;
    if nargin<2
        if nargin<1, dt=0.001; end
        tsnaps = linspace(dt,1,9);
    end
    plot_exact(tsnaps) ; % plot the exact solution surfaces if available
    for t=tsnaps
        disp(['Now processing for plot at t=',num2str(t)])          ;
        u = dynamic_2d_fem(P,Ind,...
                  @Mhandle,@bhandle,@ghandle,@hhandle,@chandle,...
                  @DBChandle,@NBChandle,u,dt,oldt,t)                ;
        k = k+1                                                     ;
        oldt = t                                                    ;
        pu = full(u)                                                ;
        figure(1)                                                   ;
        subplot(3,3,k)                                              ;
        hold on
        plot3(x,y,pu+2e-3,'ok',...
               'linewidth',0.1,...
                'markersize',1,...
                'markerfacecolor','black'); 
                text(0,0,1.9,['t= ',num2str(t)])                    ;
        hold off
        plot_errors(x,y,t,u,k)  ; % if exact solution is available
    end 
%
% =======================================================================

    function M = Mhandle(x,y,t)
            M = 2*eye(2)                        ;
    function b=bhandle(x,y,t)
            b = [0;0]                           ;
    function g=ghandle(x,y,t)
            g = -3                              ;
    function [h]=hhandle(x,y,t)
            r   = exp(2-2*x-y)                  ;
            s   = exp( 8*( (x-4/5).^2 - y ) )   ;
            phi = 8./(1+20*r)                   ;
            psi = 1./(1+5*s)                    ;
            qa  = 32000*r.^2./(1+20*r).^3     ...
                    - 800*r./(1+20*r).^2        ;
            qb  = (3200+50*(16*x-64/5).^2).*s.^2./(1+5*s).^3 -... 
                  (400 + 5*(16*x-64/5).^2).*s./(1+5*s).^2;
            sig1 = (phi - 2*qa - psi + 2*qb)    ;
            sig2 = (3*psi - 2*qb)               ;
            h = exp(-2*t)*sig1 + sig2           ;
    function c=chandle(x,y,t)
            c = 1                               ;    
    function [DBC, DBCval] = DBChandle(P,t)
        x           = P(1,:)                    ;
        y           = P(2,:)                    ;
        Nnodes      = length(x)                 ;
        DBC         =   (x==0) | (x==1) |...
                        (y==0) | (y==1)         ; % north, south,
        DBCval  = sparse(Nnodes,1)              ;
        xdbc    = x(DBC==1)                     ; % x @ Dirich nodes
        ydbc    = y(DBC==1)                     ; % y @ Dirich nodes
        phi     = 8./(1 + 20*exp(2-2*xdbc-ydbc));
        psi     = 1./(1 + 5*exp( ...
                    8*((xdbc-4/5).^2-ydbc)))    ;
        f       = exp(-2*t)                     ;        
        DBCval(DBC==1) = f*phi+(1-f)*psi        ; % get Dirich BCs
    function [NBC,NBCval] = NBChandle(P,t)
        Nnodes      = size(P,2)                 ;
        NBC         = sparse(Nnodes,1)          ;
        NBCval      = sparse(Nnodes,1)          ; % get Neumann BCs
    function u = get_initial_condition(x,y)
        u           = 8./(1+20*exp(2-2*x-y))'   ;
    function u = uexact(x,y,t)
        f = exp(-t/0.5);
        u1 = 1./( 1 + 5*exp( 8*((x-4/5).^2 - y) ) );
        u2 = 8./(1 + 20*exp(2-2*x-y));
        u = (f)*u2 + (1-f)*u1;
    function plot_exact(tt)
        xrange  = linspace(0,1,21)              ;
        yrange  = linspace(0,1,21)              ;
        [x,y]   = meshgrid(xrange, yrange)      ;
        k       = 0                             ;
        figure(1)                               ;
        for t=tt
            u   = uexact(x,y,t)                 ;
            k   = k+1                           ;
            subplot(3,3,k,'fontsize',8)         ;
            surfl(x,y,u)                        ;
            text(0,0,1.9,['t= ',num2str(t)])    ;
            shading interp                      ;
            colormap gray                       ;
            axis([0 1 0 1 0 1])                 ;
        end
    function plot_errors(x,y,t,u,k)
            pu   = full(u)                      ;
            figure(2)                           ;
            ex_u = uexact(x,y,t)                ;
            er_u = pu-ex_u'                     ;
            subplot(3,3,k)                      ;
            plot3(x,y,er_u,'ok',...
                'linewidth',0.1,...
                'markersize',1,...
                'markerfacecolor','black')      ;
            axis([0 1 0 1 -0.03 0.03])          ;
            text(0,0,0.09,['t= ',num2str(t)])   ;