function    [t,y] = fehlberg45(f,t0,tfinal,y0,dt,tolerance)
%   
% function    [t,y] = fehlberg45(f,t0,tfinal,y0,dt,tolerance)
% =========================================================
%
%       f   = function handle for derivative function, f(t,y)
%             ( use @fname )
%       t0  = initial time   ;      tfinal  = final time
%       y0  = initial value  ;      dt      = initial time step
%
%       Optional: tolerance ( default: 1e-8 )

% (c) 2006, 2010 by Tomas Co  @ Michigan Technological University

    if nargin<5         ;  tolerance = 1e-8    ;  end                   ;
    thmax   = 2         ;  thmin   = 0.5       ;  hmax = 10*dt          ;    
    t       = t0        ;  tk      = t0        ;  h    = dt             ;
    yk      = y0        ;  y       = y0'                                ;                          
    a1  = 0             ;  a2  = 1/4           ;  a3  = 3/8             ;  
    a4  = 12/13         ;  a5  = 1             ;  a6  = 1/2             ;
    b21 = 1/4           ;  b31 = 3/32          ;  b32 = 9/32            ;
    b41 = 1932/2197     ;  b42 = -7200/2197    ;  b43 = 7296/2197       ;
    b51 = 439/216       ;  b52 = -8            ;  b53 = 3680/513        ;  
    b54 = -845/4104     ;  b61 = -8/27         ;  b62 = 2               ;
    b63 = -3544/2565    ;  b64 = 1859/4104     ;  b65 = -11/40          ;
    c1  = 25/216        ;  c2  = 0             ;  c3  = 1408/2565       ;
    c4  = 2197/4104     ;  c5  = -1/5          ;  c6  = 0               ;
    d1  = 16/135        ;  d2  = 0             ;  d3  = 6656/12825      ;
    d4  = 28561/56430   ;  d5  = -9/50         ;  d6  = 2/55            ;
    while tk < tfinal                                                   ;
        h       = min(h,tfinal-tk)                                      ;
        delta1  = h*f(tk,yk)                                            ;
        delta2  = h*f(tk+a2, yk + b21*delta1)                           ;
        delta3  = h*f(tk+a3, yk + b31*delta1 + b32*delta2)              ;
        delta4  = h*f(tk+a4, yk + b41*delta1 + b42*delta2 + b43*delta3) ;
        delta5  = h*f(tk+a5, yk + b51*delta1 + b52*delta2 + b53*delta3... 
                                + b54*delta4)                           ;
        delta6  = h*f(tk+a6, yk + b61*delta1 + b62*delta2 + b63*delta3...
                                + b64*delta4 + b65*delta5)              ;
        zk      = yk+(c1*delta1 + c2*delta2  + c3*delta3  + c4*delta4 ...
                                + c5*delta5  + c6*delta6 )              ;
        wk      = yk+(d1*delta1 + d2*delta2  + d3*delta3  + d4*delta4 ...
                                + d5*delta5 + d6*delta6 )               ;
        tau     = max(abs(wk-zk))/h                                     ;
        if tau <= tolerance                                             ;
                  yk  = zk      ;   y   = [y;yk']                       ;
                  tk  = tk + h  ;   t   = [t;tk]                        ;
        end                                                             ;
        if tau==0                                                       ;  
		      theta = thmax                                             ;
        else                                                            
                  theta = (tolerance/(2*tau))^(1/4)                     ;
                  theta = min(thmax,max(theta,thmin))                   ;
                  h     = min(hmax,theta*h)                             ;
        end
    end