function  [Q,W,r,Qinv,Winv]   = gauss_jordan(A,tol)
%
%   function    [Q,W,r,Qinv,Winv] = gauss_jordan(A,tol)
%   ===================================================   
%   
%       Uses Gauss-Jordan method with pivoting
%       to determine matrices Q and W such that
%
%                (  I  |  0  )
%       Q A W =  ( ----+---- )
%                (  0  |  0  )
%
%       Optional input:
%           tol = tolerance ( default:1e-10 )
%       Optional output:
%           r       = rank of A (based on tol)
%           Qinv    = inverse of Q
%           Winv    = inverse of Q

%   (c) 2006 by Tomas Co
%   @ Michigan Technological University
%

    if nargin<2
        tol = 1e-10                                 ;
    end
    [N,M]   = size(A)                               ;
    spA     = issparse(A)                           ;
    A       = sparse(A)                             ;
    B       = A                                     ;
    Q       = speye(N)                              ;
    W       = speye(M)                              ;
    Qinv    = speye(N)                              ;
    Winv    = speye(M)                              ;
    K       = min(N,M)                              ;   

    for r=1:K
        if  max(max(abs(B)))<tol
            r           = r-1                       ;
            [Q,W]       = makefull(Q,W,spA)         ;
            return
        end
        [Qk,Wk,Qki,Wki] = reduce(B)                 ;
        QQ              = speye(N)                  ;
        WW              = speye(M)                  ;
        QQi             = speye(N)                  ;
        WWi             = speye(M)                  ;
        QQ(r:end,r:end) = Qk                        ;
        WW(r:end,r:end) = Wk                        ;
        QQi(r:end,r:end)= Qki                       ;
        WWi(r:end,r:end)= Wki                       ;
        Q               = QQ*Q                      ;
        W               = W*WW                      ;
        Qinv            = Qinv*QQi                  ;
        Winv            = WWi*Winv                  ;
        r               = r+1                       ;
        H               = Qk*B*Wk                   ;
        H(abs(H)<tol)   = 0                         ;
        B               = H(2:end,2:end)            ;
    end
    r = r-1                                         ;
    [Q,W]               = makefull(Q,W,spA)         ;
%    
%---------------------------------------------------------    
%
    function [Ec_left,Ec_right,Ec_linv,Ec_rinv] = reduce(A)
    %======================================================
    %
        [N,M]           = size(A)                   ;
        Ileft           = speye(N)                  ; 
        Iright          = speye(M)                  ;
    %
    %   find imax and jmax
    %   ------------------
        [y,i]           = max(abs(A))               ;
        [y,jmax]        = max(y)                    ;
        if N>1
            imax        = i(jmax)                   ;
        else
            imax        = 1                         ;
        end
        Amax            = A(imax,jmax)              ;
    %
    %   obtain vector Ec_right and Ec_rinv
    %   ----------------------------------
        v               = -A(imax,:)/Amax           ;
        vv              = -v                        ;
        v(jmax)         = 1                         ;
        vv(jmax)        = 1                         ;
        Ec_right        = Iright                    ;
        Ec_right(jmax,:)= v                         ;
        Ec_right(:,[1,jmax]) =  ...
            Ec_right(:,[jmax,1])                    ;
        Ec_rinv        = Iright                     ;
        Ec_rinv(jmax,:)= vv                         ;
        Ec_rinv([1,jmax],:) =  ...
            Ec_rinv([jmax,1],:)                     ;
    %
    %   obtain vector Ec_left and Ec_linv
    %   ---------------------------------
        w               = -A(:,jmax)/Amax           ;
        ww              = -w                        ;
        w(imax)         = 1/Amax                    ;
        ww              = ww*Amax                   ;
        Ec_left         = Ileft                     ;
        Ec_left(:,imax) = w                         ;
        Ec_left([1,imax],:) = ...
            Ec_left([imax,1],:)                     ;
        Ec_linv         = Ileft                     ;
        Ec_linv(:,imax) = ww                        ;
        Ec_linv(:,[1,imax]) = ...
            Ec_linv(:,[imax,1])                     ;
                                
    function [Q,W] = makefull(Q,W,spA)
    %
    %   if A was full, then make L and U full
    %   =====================================
        if ~spA
            Q           = full(Q)                   ;
            W           = full(W)                   ;
        end