function    [t,y] = glirk(f,t0,tfinal,y0,dt,tolerance,newttol)
%   
%   function [t,y] = g1irk(f,t0,tfinal,y0,dt,tolerance,newttol)
%   ===========================================================
%
%       Gauss - Legendre implicit Runge-Kutta method
%
%       f       = function handle ( use @fname )
%       t0      = initial time  ; tfinal  = final time
%       y0      = initial value ; dt      = time step
%       optional: tolerance ( for local error,  default: 1e-6 ) 
%                 newttol   ( for newton error, delault: 1e-6 )
%

%   (c) 2006, 2010 by Tomas Co, 
%   @ Michigan Technological University

    if nargin < 6;       tolerance   = 1e-6    ;    end         ;
    if nargin < 7;       newttol     = 1e-6    ;    end         ;
    kt      = 1/(2^5-2)                                         ;
    thmax   = 5              ;   thmin   = 0.5                  ;  
    hmax    = 10*dt          ;   dymin   = 1e-4                 ;
    t       = [t0]           ;   y       = [y0']                ;
    tk      = t0             ;   yk      = y0                   ;
    n       = length(y0)     ;   E       = eye(n)               ;
    b11     = 1/4            ;   b12     = 1/4 - sqrt(3)/6      ;
    b22     = 1/4            ;   b21     = 1/4 + sqrt(3)/6      ;
    a1      = 1/2-sqrt(3)/6  ;   a2      = 1/2+sqrt(3)/6        ;
    c1      = 1/2            ;   c2      = 1/2                  ;
    h       = dt             ;   dy      = min(dymin,h)         ;
%                                                               ;
    while tk < tfinal                                           ;
        h       = min(h,tfinal-tk)                              ;
        htemp   = h   ;    ytemp   = yk   ;   ttemp   = tk      ;
        for sw  = 1:3                                           ;
            dy  = min(dymin,h)                                  ;
            if sw == 3                                          ;
                tk = ttemp ; yk = ytemp ; h = 2*htemp           ;
            end                                                 ;
            delta1  = zeros(n,1)      ;   delta2  = zeros(n,1)  ;
            delta   = [delta1;delta2] ;   dd      = 1           ;

            while dd > newttol                                  ;
                dg1 = f( tk+a1*h, yk+b11*delta1+b12*delta2 )    ;
                dg2 = f( tk+a2*h, yk+b21*delta1+b22*delta2 )    ;
                g1  = delta1 - h*dg1  ;   g2  = delta2 - h*dg2  ;
                G   = [g1;g2]         ;   dd  = norm(G)         ;
                for j=1:n
                    yy  = yk+ b11*(delta1+E(:,j)*dy)+ b12*delta2;
                    dG([1:n],j) = f(tk+a1*h,yy)- dg1            ;
                    yy  = yk+ b12*(delta2+E(:,j)*dy)+ b11*delta1;
                    dG([1:n],j+n) = f(tk+a1*h,yy)- dg1          ;
                    yy  = yk+ b21*(delta1+E(:,j)*dy)+ b22*delta2;
                    dG([1:n]+n,j) = f(tk+a2*h,yy)- dg2          ;
                    yy  = yk+ b22*(delta2+E(:,j)*dy)+ b21*delta1;
                    dG([1:n]+n,j+n) = f(tk+a2*h,yy)- dg2        ;
                end                                             ;
                J      = eye(2*n) - (h/dy)*dG                   ;
                delta  = delta - pinv(J)*G                      ;
                delta1 = delta([1:n])                           ;
                delta2 = delta([n+1:2*n])                       ;
            end                                                 ;
            tk = tk + h ;   yk = yk + c1*delta1 + c2*delta2     ;
            if sw==1              ;  nk = yk                    ;
                elseif sw==2    ;  zk = yk                    ;
                  else            ;  wk = yk                    ;
            end                                                 ;
        end                                                     ;
        yk    = nk     ;    y   = [y;yk']   ;   h   = htemp     ;
        tk    = ttemp  ;    tk  = tk + h    ;   t   = [t;tk]    ;
        tau   = max( abs(zk-wk) )*kt                            ;
        theta = ( 0.8*tolerance/tau )^(1/5)                     ;
        theta = min( thmax, max(theta,thmin) )                  ;
        h     = theta*h                                         ;
    end                                                         ;
%                                                               ;    