function    [x,f] = levenmarq( Fx, x0, data)
%                                                                        ;
%   function    [x,f] = levenmarq( Fx, x0, data)                         ;
%   ============================================                         ;
%   Levenberg-Marquardt nonlinear least squares solution                 ;
%       x0  : initial guess                                              ;
%       Fx  : handles for function vector                                ;
%      data : data used in nonlinear least squares                       ;

%   (c) 2007 by Tomas Co                                                 ;
%   @ Michigan Technological Universtity                                 ;

eps = 1e-15     ;   GTol = (eps)^(0.33)         ;   StpTol = GTol^2      ;
MXstp = 300    ;   MXiter = 500                ;   nmax = 0             ;
xc = x0         ;   r = feval(Fx,xc,data)       ;   fc = r'*r/2          ;
J = Jac(Fx,xc,data)                             ;   gc = r'*J            ;
chk = CKend( xc,fc,gc,GTol,nmax )                                        ;
if chk == 1     ;   x=xc        ;   f=fc        ;   return      ; end    ;
not_done=true   ;   iter=0      ;   del=-1      ;   mu=0                 ;
while not_done                                                           ;
  iter = iter+1                                                          ;
  [x,r,f,g,mu,del,NoCh,MXed] = update(Fx,data,xc,del,mu,MXstp,StpTol)    ;
  [chk,nmax] = CKend(x,f,g,GTol,nmax,xc,iter,NoCh,MXed,StpTol,MXiter)    ;
  if chk>0      ;   not_done = false                            ; end    ;
  xc = x                                                                 ;
  [iter,f,x']                                                          
end                                                                      ;


function [chk,nmax] = CKend( x,f,g,GTol,nmax,xc,iter,NoCh,...
          MXed,StpTol,MXiter )                                           
% =======================================================================;
%       check for termination                                            ;
%                                                                        ;
v1=max(abs(x),1);           v3=max(abs(g).*v1'/max(abs(f),1))            ; 
if  v3 <= GTol*1e-3                                                      ;
    chk = 1     ;   disp(' Tolerance met for gradient') ;   return       ;
else                chk = 0                                              ;
end                                                                      ;
if nargin > 5                                                            ;
v4=max(abs(x-xc)./v1)                                                    ;
    if      NoCh;          chk=3    ; disp('No change in global steps ') ;
    elseif  v4 <= StpTol;  chk=2    ; disp('Tolerance met for step size');  
    elseif  iter >MXiter;  chk=4    ; disp(' Number iterations exceeded');
    elseif  MXed        ;  nmax=nmax + 1                                 ;
        if nmax > 4; chk =5;  disp( 'Maxed 5 consecutive times') ;  end  ;
    else                   nmax = 0                                      ;
    end                                                                  ;
end                                                                      ;
        
function    J   = Jac(Fx,x,data)                                         
% ==============================                                         ;
%   Finite difference approximation of Jacobian                          ;
%                                                                        ;
d=eps^(0.5) ;   n=length(x)             ;   h=max(abs(x),1)*d            ; 
J=[]        ;   f=feval(Fx,x,data)                                       ;   
for i=1:n   ;   xx=x                    ;   xx(i)=x(i)+h(i)              ;   
                ff=feval(Fx,xx,data)    ;   J=[J,(ff-f)/h(i)]            ;
end                                                                      ;

function [x,r,f,g,mu,del,NoCh,MXed]=update(Fx,data,xc,del,mu,MXstp,StpTol) 
%=========================================================================
%    More update
%    
not_done=true               ;   rcode=4             ;  alpha=1e-4        ;   
NoCh=false                  ;   r=feval(Fx,xc,data) ;  J=Jac(Fx,xc,data) ;
gc=r'*J                     ;   Hc=J'*J             ;  fc=(r'*r)/2       ;   
[sN,NLen,dqN]=solveS(J,r,0) ;   MXed=false                               ;
if del==-1  ;       mu=0    ;   a=gc*gc'            ;  b=gc*Hc*gc'       ; 
    CPlen=norm((a/b)*(gc')) ;   del=min(CPlen,MXstp)                     ;   
end                                                                      ;
delOld = del                ;   qN=NLen - del                            ;
while not_done                                                           ;
    if  NLen < 1.1*del      ;   Nused=true          ;  s=sN              ;   
        ns=NLen             ;   del=min(del,NLen)   ;  mu=0              ;
    else                        Nused=false         ;   mu_not_done=true ;
        mu=mu*delOld/del    ;   Lo=-qN/dqN          ;   Hi=norm(gc)/del  ;
        while mu_not_done                                                ;
            if (mu<=Lo)&&(mu>=Hi) ;  mu=max(sqrt(Hi*Lo),1e-3*Hi) ;   end ;
            [s,ns,dq]=solveS(J,r,mu)                ;  q=ns-del          ;
            if ((ns>=0.9*del)&&(ns<=1.1*del))||(Hi<Lo);mu_not_done=false ;
            else                mu=mu-(ns/del)*q/dq ;  Lo=max(Lo,mu-q/dq);
                    if q<0  ;   Hi=min(Hi,mu)                   ;   end  ;
            end                                                          ;
        end                                                              ;
    end                                                                  ;
%           update trust region                                          ;
%           -------------------                                          ;
    delOld=del      ;   x=xc+s      ;   r=feval(Fx,x,data)  ;   f=r'*r/2 ;   
    df=f-fc         ;   slope0=gc*s ;   dff=alpha*slope0                 ;
    if (rcode~=3)   ;   f_old   = 0                             ;   end  ;
    if (rcode==3)&&( (f>=f_old)||(df>dff) )                              ;
        rcode=0     ;   not_done=false                                   ;   
        x=x_old     ;   f=f_old     ;   del=del/2                        ;
    elseif df >= dff                                                     ;
        ndx = max( abs(s)./max(abs(x),1) )                               ;
        if ndx<StpTol;      x=xc ; f=fc ; NoCh=true ; not_done=false     ;
        else                rcode=2 ; del_m=-(slope0/2)/(df-slope0)*ns   ;
            del=min( max(0.1*del,del_m), del/2 )                         ;
        end                                                              ;
    else       df_p=slope0+ s'*Hc*s/2                                    ; 
        cond1=(~Nused)      ;   cond2=(del<=0.99*MXstp)                  ;      
        cond3=((abs(df_p-df)<=(abs(df)/10))|(df<=slope0))                ;
        if (rcode~=2) && cond1 && cond2 && cond3                         ;
                rcode=3 ;  x_old=x ;  f_old=f ;  del=min(2*del,MXstp)    ;
        else               NoCh=false         ;  not_done=false          ;
            if ns>0.99*MXstp     ;      MXed=true               ;   end  ;
            if df >= df_p/10     ;      del=del/2                        ; 
            elseif df<=0.75*df_p ;      del = min(del*2,MXstp)           ;            
            end                                                          ;
        end                                                              ;
    end                                                                  ;
    r=feval(Fx,x,data)  ;   f=r'*r/2    ;   J=Jac(Fx,x,data)  ;  g=r'*J  ;
    [sN,NLen,dqN]=solveS(J,r,0)         ;   qN = NLen-del                ;
end                                                                      ;

function    [s,ns,dq] = solveS(J,F,mu)                                   
%=====================================                                   ;
n=size(J,2) ;   b = -J'*F                                                ;
if mu>0     ;   A=[J;sqrt(mu)*eye(n)]   ;   else       A=J     ;   end  ;
L=zeros(n,n);   v=zeros(n,1)            ;   H=A'*A                       ;
MXoff=max(1,max(abs(diag(H))))          ;   MNv=eps^(1/2)*MXoff          ;
for j=1:n   ;   jn=j:n                  ;   j1=1:j-1                     ;
        if  j>1     ;   v=L(jn,j1)*L(j,j1)'                     ;   end  ;
        v=H(jn,j)-v ;   a=sqrt(max([(v.*v)/MXoff;MNv;v(1)]))             ;   
        v=v/a       ;   v(1)=a          ;   L(jn,j)=v                    ;
end                 ;   y=b(1)/L(1,1)                                    ;
for i = 2:n         ;   y =[y;(b(i)-L(i,1:i-1)*y)/L(i,i)]   ;  end       ;
R = L'              ;   s = y(n)/R(n,n)                                  ;
for i=n-1:-1:1      ;   s =[(y(i)-R(i,i+1:n)*s)/R(i,i);s]   ;  end       ;
if  nargout > 1                                                          ;
    b = s           ;   y = b(1)/L(1,1)                                  ;
    for i = 2:n     ;   y =[y;(b(i)-L(i,1:i-1)*y)/L(i,i)]   ;  end       ;
    dq = y(n)/R(n,n)                                                     ;
    for i=n-1:-1:1  ;   dq =[(y(i)-R(i,i+1:n)*dq)/R(i,i);dq];  end       ;
    ns = norm(s)    ;   dq = -(s'*dq)/ns                                 ;
end                                                                      ;