function  [U,x,y] = lin2D_centralfd_dirich(K,N)
%
% function  [U,x,y] = lin2D_centralfd_dirich(K,N)
% ===============================================
%
% Finite difference solution of 
% a second order linear PDE given by:
%
%     mu_xx uxx + mu_xy uxy + mu_yy uyy
%         + beta_x ux + beta_y uy
%             + zeta u + eta    =   0
%
%   under Dirichlet boundary conditions
%   based on central difference formulas
%
% where,
%   K,N = number of grid in x, y, resp.
%   x,y = mesh grid points
%   U   = solution at the mesh grid points
%
% Note: user needs to modify the functions 
% near the bottom of the file to customize 
% coefficients and boundary conditions:
%
%     get_coefs(x,y) to define mu_xx, ... etc.
%     bc_at_x_equal_0(y) to define u(0,y)
%     bc_at_x_equal_1(y) to define u(1,y)
%     bc_at_y_equal_0(x) to define u(x,0)
%     bc_at_y_equal_0(x) to define u(x,1)

% (c) 2011 by Tomas Co
% @ Michigan Tech University

x0 = 0 ; x1 = 1                                     ;
y0 = 0 ; y1 = 1                                     ;
KN = K*N                                            ;
oK = ones(K,1)                                      ;
oN = ones(N,1)                                      ;
dx = (x1-x0)/(K+1)                                  ;
dy = (y1-y0)/(N+1)                                  ;
xrange = x0+dx*(1:K)                                ;
yrange = y0+dy*(1:N)                                ; 
[y,x] = meshgrid(yrange,xrange)                     ;
R = sparse(KN,KN)                                   ;
f = sparse(KN,1)                                    ;

% -------------------------------------------------
% Customize the coefficients in the function called
% "get_coefs" near bottom of this file.
% -------------------------------------------------
[mu_xx,mu_xy,mu_yy,beta_x,...
    beta_y,zeta,eta] = get_coefs(x,y)               ;

% -------------------------------------------------
% Customize the boundary conditions in the functions 
% called "bc_at_x_equal_0", ... etc.
% near bottom of this file. 
% -------------------------------------------------

u_0y = zeros(1,N);
u_1y = zeros(1,N);
u_x0 = zeros(1,K);
u_x1 = zeros(1,K);
for n=1:N
    u_0y(n) = bc_at_x_equal_0(yrange(n))            ;
    u_1y(n) = bc_at_x_equal_1(yrange(n))            ;
end
for k=1:K
    u_x0(k) = bc_at_y_equal_0(xrange(k))            ;
    u_x1(k) = bc_at_y_equal_1(xrange(k))            ;
end
u_00 = (bc_at_x_equal_0(y0) + bc_at_y_equal_0(x0))/2;
u_01 = (bc_at_x_equal_0(y1) + bc_at_y_equal_1(x0))/2;
u_10 = (bc_at_x_equal_1(y0) + bc_at_y_equal_0(x1))/2;
u_11 = (bc_at_x_equal_1(y1) + bc_at_y_equal_1(x1))/2;
% =======================================

if  nnz(mu_xx),
    mu_xx = make_dvec(mu_xx)                        ;
    D2x = spdiags(oK*[1 -2 1],[-1 0 1],K,K )/dx^2   ;
    B2x = sparse(K,N)                               ;
    B2x(1,:) = u_0y/dx^2                            ;
    B2x(end,:) = u_1y/dx^2                          ;
    R = R + mu_xx*kron(speye(N),D2x)                ;
    f = f + mu_xx*B2x(:)                            ;
end

if  nnz(mu_yy),
    mu_yy = make_dvec(mu_yy)                        ;
    D2y = spdiags( oN*[1 -2 1],[-1 0 1],N,N )/dy^2  ;
    B2y = sparse(K,N)                               ;
    B2y(1,:) = u_x0/dy^2                            ;
    B2y(end,:) = u_x1/dy^2                          ;
    B2y = B2y'                                      ;
    R = R + mu_yy*kron(D2y,speye(K))                ;
    f = f + mu_yy*B2y(:)                            ;
end

if  nnz(mu_xy),
    mu_xy = make_dvec(mu_xy)                        ;
    D1x = spdiags( oK*[-1 0 1],[-1 0 1],K,K )/(2*dx);
    D1y = spdiags( oN*[-1 0 1],[-1 0 1],N,N )/(2*dy);
    B1x1y = sparse(N,K)                             ;
    B1x1y(1,:) = -[u_0y(2:N),u_01/2]+...
                    [u_00/2,u_0y(1:N-1)]            ;
    B1x1y(end,:) = [u_1y(2:N),u_11/2]-...
                    [u_10/2,u_1y(1:N-1)]            ;
    B1x1y(:,1) = -[u_x0(2:K),u_10/2]'+ ... 
            [u_00/2,u_x0(1:K-1)]'+B1x1y(:,1)        ;
    B1x1y(:,end) = [u_x1(2:K),u_11/2]'- ...
            [u_01/2,u_x1(1:K-1)]' + B1x1y(:,end)    ;    
    B1x1y = B1x1y/(4*dx*dy)                         ;    
    R = R + mu_xy*kron(D1y,D1x)                     ;
    f = f + mu_xy*B1x1y(:)                          ;
end

if  nnz(beta_x),
    beta_x = make_dvec(beta_x)                      ;
    D1x = spdiags( oK*[-1 0 1],[-1 0 1],K,K )/(2*dx);
    B1x = sparse(K,N)                               ;
    B1x(1,:) = -u_0y/(2*dx)                         ;
    B1x(end,:) = u_1y/(2*dx)                        ;
    R = R + beta_x*kron(speye(N),D1x)               ;
    f = f + beta_x*B1x(:)                           ;
end

if  nnz(beta_y),
    beta_y = make_dvec(beta_y)                      ;
    D1y = spdiags( oN*[-1 0 1],[-1 0 1],N,N )/(2*dy);
    B1y = sparse(N,K)                               ;
    B1y(1,:) = -u_x0/(2*dy)                         ;
    B1y(end,:) = u_x1/(2*dy)                        ;
    B1y = B1y'                                      ;
    R = R + beta_y*kron(D1y,speye(K))               ;
    f = f + beta_y*B1y(:)                           ;
end

if  nnz(zeta)>0,
    if isscalar(zeta)
        zeta = zeta*ones(K,N)                       ;
    end
    zeta=spdiags(zeta(:),0,KN,KN)                   ;
    R = R + zeta                                    ;
end

if nnz(eta)>0
    f = f + sparse(eta(:))                          ;
end

v = R\(-f)                                          ;
U = full(reshape(v,K,N))                            ;

function dv = make_dvec(w)
    if isscalar(w)
        dv = w                                      ;
    else
        q = numel(w)                                ;
        dv = spdiags(w(:),0,q,q)                    ;
    end

function [mu_xx,mu_xy,mu_yy,beta_x,...
    beta_y,zeta,eta] = get_coefs(x,y)
%
%  Here x and y are treated as meshgrid points
%  so used elementwise operations such as ".*",
%  ".^", ... etc. 
%  For example: eta = x.*exp(x.^2+y.^2);
%
%  If coefficient is constant, just define as scalar
%
        mu_xx   = 1                                 ;
        mu_xy   = 2                                 ;
        mu_yy   = 1                                 ;
        beta_x  = 1                                 ;
        beta_y  = -2                                ;
        zeta    = 0                                 ;
        eta     = -9*y                              ;

%  The functions below are for the 
%  Dirichlet boundary conditions.  
%  Here, x and y
%  are treated as scalars.
%
    function u_0y = bc_at_x_equal_0(y)
        u_0y  = -2*y^2                              ;
    function u_1y = bc_at_x_equal_1(y)
        u_1y  = 1 + y - 2*y^2                       ;
    function u_x0 = bc_at_y_equal_0(x)
        u_x0  = x^2                                 ;
    function u_x1 = bc_at_y_equal_1(x)
        u_x1  = x^2 + x - 2                         ;