function    u = linear_2d_supg(P,Ind,...
                    DBC,DBCval,NBC,NBCval,...
                    RBC,RBCval,RBCmuval,...
                    Mhandle,bhandle,ghandle,hhandle)
%
%  function u = linear_2d_supg(P,Ind,...
%            DBC,DBCval,NBC,NBCval,...
%            Mhandle,bhandle,ghandle,hhandle)
%  =====================================================
%
%    PDE:
%       div( M dot grad(u)) + b dot grad(u) + gu + h = 0
%
%    P       : node positions        [=] 2 x Nnodes
%    Ind     : indices of elements   [=] Nelements x 3
%    DBC     : Dirichlet nodes,      1 = true, 0 = false
%    DBCval  : Dirichlet Values
%    NBC     : Neumann nodes,        1 = true, 0 = false
%    NBCval  : Neumann values
%    @Mhandle,@bhandle,@ghandle,@hhandle: function handles
%   (assumes that all the triangles have been indexed
%         with a counter-clockwise sequence)

%       (c) 2005, 2011 by Tomas Co
%       Michigan Technological University
%

    Nnodes          = size(P,2)                     ;
    Nelements       = size(Ind,1)                   ;
    K               = sparse(Nnodes,Nnodes)         ;
    Gamma           = sparse(Nnodes,1)              ;
    for i=1:Nelements
        I           = Ind(i,:)                      ;
        P_loc       = P(:,I)                        ;
        NBC_loc     = NBC(I)                        ;
        NBCval_loc  = NBCval(I)                     ;
        RBC_loc     = RBC(I)                        ;
        RBCval_loc  = RBCval(I)                     ;
        RBCmuval_loc= RBCmuval(I)                   ;
        [K_loc,Gamma_loc] = local_fem(P_loc,...
                    Mhandle, bhandle, ghandle,...
                    hhandle, NBC_loc, NBCval_loc,...
                    RBC_loc, RBCval_loc, RBCmuval_loc);
        K(I,I)      = K(I,I) + K_loc                ;
        Gamma(I)    = Gamma(I) + Gamma_loc          ;
    end
    epss            = 1e-13                         ;
    for i=1:Nnodes
        if DBC(i)
            alpha   = K(i,i)/epss                   ;
            K(i,i)  = alpha                         ;
            Gamma(i)= DBCval(i)*alpha               ;
        end
    end
    u               = K\Gamma                       ;
%
% ===================================================
%
    function [K_n,Gamma_n]=local_fem(p,Mhandle,...
                bhandle,ghandle,hhandle,NBC,NBCval,...
                RBC,RBCval,RBCmuval)
        D = det([1 1 1;p])                          ;
        T       = [0,-1;1,0]*p*...
                    [0,1,-1;-1,0,1;1,-1,0]/D        ;
        zeta    = [1;1;1]/3                         ;
        pstar   = p*zeta                            ;
        pxstar  = pstar(1)                          ;
        pystar  = pstar(2)                          ;
        M       = feval(Mhandle,pxstar,pystar)      ;
        b       = feval(bhandle,pxstar,pystar)      ;
        g       = feval(ghandle,pxstar,pystar)      ;
        h       = feval(hhandle,pxstar,pystar)      ;
        QNBC    = evalQNBC(NBC,NBCval,p,D)          ;        
        [Lambda,QRBC] = ...
                evalQRBC(RBC,RBCmuval,RBCval,p,D)   ;
        tau     = new_stab_tau(p,b,norm(M),g)       ;

        K_n     = (D/2)*( - Lambda + ... 
                T'*M*T - zeta*b'*T - g*zeta*zeta'...
                - tau*T'*b*b'*T )                   ;
        Gamma_n = (D/2)*(h*zeta + QNBC + QRBC...
                    + tau*(T'*b)*h )                ;
%
% ===================================================
%
  function Q = evalQNBC(NBC,NBCval,p,D)
%        
        Q       = [0;0;0]                           ;
        if sum(NBC)>1
            if sum(NBC)>2
                disp(['All 3 nodes of element ',...
                  num2str(i),...
                  ' are attached to the boundary']) ;
                return
            end
            L   = norm(p(:,NBC==1)*[1 0;-1 0])      ;
            Q   = NBCval*L/D                        ;
        end
%
% ===================================================
%
  function [Mu,Q] = evalQRBC(RBC,RBCmuval,RBCval,p,D)
%        
        Q       = [0;0;0]                           ;
        Mu      = zeros(3,3)                        ;
        if sum(RBC)>1
            if sum(RBC)>2
                disp(['All 3 nodes of element ',...
                  num2str(i),...
                  ' are attached to the boundary']) ;
                return
            end
            L   = norm(p(:,RBC==1)*[1 0;-1 0])      ;
            Q   = RBCval*L/D                        ;
            Mu  = diag(RBCmuval)*L/D                ;
        end