function  [x0,xT,q,t,x] = nbvp(x0, t0, tT, tolerance)
%
% function    [x0,xT,q,t,x] = nbvp(x0, t0, tT, tolerance)
% =======================================================
%
%       Uses nonlinear shooting to obtatin initial
%       conditions
%           x0          : initial values
%           t0          : initial time
%           tT          : final time
%           tolerance   : on norm( q(x0,xT) )
%           q           : boundary conditions
%           t,x         : simulated time and states

%   (c) 2007 by Tomas Co
%   @ Michigan Technological University

    n       = length(x0)                            ;
    M_eye   = eye(n)                                ;
    q       = 1                                     ;                                                       
    while norm(q)>tolerance
        xM0     = [ x0 ; M_eye(:) ]                 ;
        [t,xM]  = ode45( @derfunc, [t0,tT], xM0 )   ;
        xMT     = xM( end,: )'                      ;
        xT      = xMT( 1:n )                        ;
        MT      = reshape( xMT(n+1:end), n, n)      ;

        % begin customize (formulas for q,Qa,Qb)
                Qa = [ 1, 0    , 0;
                       0, xT(2), 0;
                       0, 0    , 1  ]               ; 
                Qb = [ 0, -1   , 0;
                       0, x0(2), 0;
                       0, 0    ,  1 ]               ; 
                q = [ x0(1) - xT(2) - 0.164 ;
                      x0(2)*xT(2)   - 0.682;
                      x0(3)+xT(3)   - 1.136  ]     ;       
        % end customize
        
        J       = Qa + Qb*MT                        ;
        x0      = x0 - inv(J)*q                     ;
    end
    x       = xM( :, 1:n )                          ;

function    dxM  = derfunc( t, xM )
%
% state equation: dx/dt = F(t,x)
% ------------------------------
%       dM/dt = A M
%       A     = dF/dx
%           
    
    n   = 3                                         ; %customize
    x   = xM(1:n)                                   ;
    M   = reshape(xM(n+1:end),n,n)                  ;

    % begin customize (values of dx)
        x1  = x(1)  ;   x2 = x(2)   ;   x3 = x(3)   ; 
        k1  = 10    ;   k2 = 3      ;   k3 = 1      ; 
        r   = k1*exp(-2*t)  ;  R = r*x1*x2          ; 
        A   = [ (-r*x2),    (-r*x1),    0;
                (-r*x2),    (-r*x1),    k2;
                ( r*x2),    ( r*x1),   -k2   ]      ;
        dx1 = -R + k3                               ;  
        dx2 = -R + k2*x3                            ; 
        dx3 =  R - k2*x3                            ; 
        dx  = [dx1;dx2;dx3]                         ; 
    % end customize
    
    dM  = A*M                                       ;
    dxM = [ dx; dM(:) ]                             ;