function    [x,F] = nsolve( Fx, x0, type, par )
%
%   [x,f] = nsolve( Fx, x0, type, par )                                  ;
%   ===================================                                  ;
%   Quasi-Newton solution to nonlinear set of equations                  ;
%       Fx   : handle for function vector                                ;
%       x0   : initial guess                                             ;
%       type : 1=line search (default) , 2=dogleg                        ;
%       par  : parameters (optional)                                     ;
%  (adapted from: Dennis & Schnabel,"Numerical Methods for Uncon-        ;
%  strained Optimization and Nonlinear Equations",Prentice Hall,1983  )  ;

%   (c) 2007 by Tomas Co                                                 ;
%   @ Michigan Technological Universtity                                 ;

FTol=eps^(0.75)     ;   StpTol=FTol^2   ;   MXStep=1000                  ;   
MXiter=1e4          ;   nmax=0          ;   xc=x0                        ;   
if nargin<3 ;  type=1                                           ; end    ;
if nargin>3 ;  parY = true              ;   else  parY=false    ; end    ;
if parY     ;  Fc=feval(Fx,xc,par)      ;   Jc = Jac(Fx,xc,par)          ;
else           Fc=feval(Fx,xc)          ;   Jc = Jac(Fx,xc)              ;
end                                                                      ;
gc=Fc'*Jc           ;   fc=Fc'*Fc/2     ;   not_done=true                ;   
iter=0              ;   del=-1          ;   nsolve=true                  ;   
[chk,nmax]=CKend(xc,fc,Fc,FTol,nmax)                                     ;
if chk == 1 ;  x=xc ;   F=Fc            ;   return              ; end    ;
while not_done                                                           ;
    [p,Lc]=NewtSolve(Jc,gc)             ;   iter=iter+1                  ;
    if parY                                                              ;
        if  type==1 ; [x,f,NoCh,MXed] = lines( Fx,xc,fc,gc,p,MXStep,   ...
                                                    StpTol,nsolve,par)   ;
        else          [x,f,del,NoCh,MXed] = dogleg( Fx,xc,fc,gc,Lc,p,  ...
                                         del,MXStep,StpTol,nsolve,par)   ;
        end         ; F=feval(Fx,x,par) ;       Jc = Jac(Fx,x,par)       ;
    else                                                                 
        if  type==1 ; [x,f,NoCh,MXed] = lines( Fx,xc,fc,gc,p,MXStep,   ...
                                                    StpTol,nsolve)       ;
        else          [x,f,del,NoCh,MXed] = dogleg( Fx,xc,fc,gc,Lc,p,  ...
                                         del,MXStep,StpTol,nsolve)       ;
        end         ; F=feval(Fx,x)     ;       Jc=Jac(Fx,x)             ;
    end                                                                  ;
        [chk,nmax]=CKend(x,f,F,FTol,nmax,xc,iter,NoCh,MXed,StpTol,MXiter);
        gc=F'*Jc                                                         ;       
        xc=x        ; fc=f                                               ;
        if chk>0    ; not_done = false                          ; end    ;
end                                                                      ;

function    [p,L] = NewtSolve(Jc,gc)                                     
% =================================                                      ;
n=size(Jc,2)        ; b=-gc             ; nn=n-1:-1:1                    ;
if (abs(det(Jc))<eps)||(cond(Jc)>1/eps)                                  ;
    H=Jc'*Jc        ; Hnorm=max(sum(abs(H)))                             ;
    H=H+(sqrt(n*eps)*Hnorm)*eye(n)      ; R=chol(H)                      ;
else             [Q,R]=QR_alg(Jc)                                        ;
end           ;  L=R'                   ;  y=b(1)/L(1,1)                 ;
for i=2:n     ;  y=[y;(b(i)-L(i,1:i-1)*y)/L(i,i)]               ; end    ;
p=y(n)/R(n,n) ;  for i=nn   ; p=[(y(i)-R(i,i+1:n)*p)/R(i,i);p]  ; end    ;
   
function [chk,nmax]=CKend(x,f,F,FTol,nmax,xc,iter,NoCh,MXed,StpTol,MXiter)            
%========================================================================
if  max(abs(F))<=FTol                                                    ; 
    chk=1     ;  disp('Tolerance met for Functions')  ;     return       ;
else             chk = 0                                                 ;
end                                                                      ;
if nargin>6   ;  v4=max( abs(x-xc)./max(abs(x),1) )                      ;
    if NoCh             ;  chk=3 ;   disp('No change in global steps'  ) ;
    elseif  v4<=StpTol  ;  chk=2 ;   disp('Tolerance met for step size') ;
    elseif  iter>MXiter ;  chk=4 ;   disp('Number iterations exceeded' ) ;
    elseif  MXed        ;  nmax=nmax+1                                   ;
        if nmax>4       ;  chk=5 ;   disp('Maxed 5 consecutive times'  ) ;
        end                                                              ;
    else                   nmax=0                                        ;
    end                                                                  ;
end                                                                      ;

function    J   = Jac(func,x,par)                                        
% ===============================                                        ;
d=eps^(0.5)  ;  n=length(x)         ;   h = max(abs(x),1)*d     ; J=[]   ;
if nargin>2  ;  parY=true           ;   else    parY=false      ; end    ;
if parY      ;  f=feval(func,x,par) ;   else    f=feval(func,x) ; end    ;
for i=1:n    ;  xx=x                ;   xx(i)=x(i)+h(i)                  ;
    if parY  ; ff=feval(func,xx,par);   else    ff=feval(func,xx); end   ;
    J=[J,(ff-f)/h(i)]                                                    ;
end                                                                      ;