function [U,r,th,x,y] = poisson2d_sphere_dirich(K,N)
%
% function [U,x,y] = poisson2d_sphere_dirich(K,N)
% ===============================================
%
%   Obtains finite difference solution to 2D Poisson equation
%
%                   del^2 u = f(r,theta)
%
%   in spherical coordinates under azimuthal symmetry
%   ( using central difference approx )
%
%   where K  is the number of internal radial grids
%         N  is the number of angular grids
%         U  is the solution matrix 
%            (including boundary condtion and closed domain)
%   Optional:
%         r, th are the polar meshgrid points
%         x, y  are the rectangular meshgrid points
%             useful when using surf, surfl, point3, etc.
%
%   Modify the bottom part of file to customize:
%        a) Dirichlet boundary condition in function "bc"
%        b) forcing function in function "forcefcn"

%  (c) 2011 by Tomas Co
%  @ Michigan Technological University

    dTh         = 2*pi/N                                            ;
    dr          = 1/(K+1)                                           ;
    rrange      = (1:K)*dr                                          ;
    thrange     = ((1:N)-1)*dTh + dTh/3                             ;

    D2          = spdiags(ones(K,1)*[1 -2 1],[-1 0 1],K,K)/(dr^2)   ;    
    D1          = spdiags(ones(K,1)*[-1 1],[-1 1],K,K)/(2*dr)       ;
    D2p         = spdiags(ones(N,1)*[1 -2 1],[-1 0 1],N,N)/(dTh^2)  ;
    D2p(1,end)  = 1/dTh^2                                           ;
    D2p(end,1)  = 1/dTh^2                                           ;
    D1p         = spdiags(ones(N,1)*[-1 1],[-1 1],N,N)/(2*dTh)      ;
    D1p(1,end)  = -1/(2*dTh)                                        ;
    D1p(end,1)  = 1/(2*dTh)                                         ;

    V           = 2*spdiags(rrange'.^(-1),0,K,K)                    ;
    W           = spdiags(rrange'.^(-2),0,K,K)                      ;
    Q           = spdiags((cos(thrange)./sin(thrange))',0,N,N)      ;
    
    B2          = sparse(K,N)                                       ;
    B1          = sparse(K,N)                                       ;
    F           = sparse(K,N)                                       ;
    for n=1:N
        B1(end,n) = bc(thrange(n))/(2*dr)                           ;
        B2(end,n) = bc(thrange(n))/dr^2                             ;
        for k=1:K
            F(k,n) = forcefcn(rrange(k),thrange(n))                 ;
        end
    end
    B           = B2 + V*B1                                         ;

    A           = kron(speye(N),D2+V*D1)+kron(D2p+Q*D1p,W)          ;
    b           = F(:)-B(:)                                         ;
    U           = A\b                                               ;
    U           = full(reshape(U,K,N))                              ;
    U           = [U,U(:,1)]                                        ;
    [th,r]      = meshgrid( (0:N)*dTh + dTh/3, dr*((1:K)-1/2))      ;
    x           = r.*cos(th)                                        ;
    y           = r.*sin(th)                                        ;

    function g = bc(theta)
    % Customize boundary condition at r=1
        g = 3+ sin(theta);

    function f = forcefcn(r,theta)
    % Customize forcing function f(r,theta)
        f = (5-4*(cos(theta)^2))/sin(theta);